/*
 * class MPZ methods
 */

inline MPZ::MPZ()
{
  mpz_init(rep);
}

inline MPZ::~MPZ()
{
  mpz_clear(rep);
}

inline MPZ::operator const __mpz_struct *() const
{
  return rep;
}

inline MPZ &MPZ::set(const MPZ &op)
{
  if (&op == this) {
    return *this;
  }
  mpz_set(rep, op.rep);
  return *this;
}

inline MPZ &MPZ::set(unsigned long int op)
{
  mpz_set_ui(rep, op);
  return *this;
}

inline MPZ &MPZ::set(unsigned int op)
{
  return set(static_cast<unsigned long int>(op));
}

inline MPZ &MPZ::set(signed long int op)
{
  mpz_set_si(rep, op);
  return *this;
}

inline MPZ &MPZ::set(signed int op)
{
  return set(static_cast<signed long int>(op));
}

inline MPZ &MPZ::set(double op)
{
  mpz_set_d(rep, op);
  return *this;
}

inline MPZ &MPZ::set(const MPQ &op)
{
  mpz_set_q(rep, op.rep);
  return *this;
}

inline MPZ &MPZ::set(const MPF &op)
{
  mpz_set_f(rep, op.rep);
  return *this;
}

inline MPZ &MPZ::set(const char *str, int base) throw(InvalidNumberException)
{
  if (mpz_set_str(rep, str, base) != 0) {
    throw InvalidNumberException();
  }
  return *this;
}

inline MPZ &MPZ::operator =(const MPZ &op)
{
  return set(op);
}

inline MPZ &MPZ::operator =(unsigned long int op)
{
  return set(op);
}

inline MPZ &MPZ::operator =(unsigned int op)
{
  return set(op);
}

inline MPZ &MPZ::operator =(signed long int op)
{
  return set(op);
}

inline MPZ &MPZ::operator =(signed int op)
{
  return set(op);
}

inline MPZ &MPZ::operator =(double op)
{
  return set(op);
}

inline MPZ &MPZ::operator =(const MPQ &op)
{
  return set(op);
}

inline MPZ &MPZ::operator =(const MPF &op)
{
  return set(op);
}

inline MPZ &MPZ::operator =(const char *str) throw (InvalidNumberException)
{
  return set(str);
}

inline MPZ &MPZ::swap(MPZ &rop2)
{
  mpz_swap(rep, rop2.rep);
  return *this;
}

inline MPZ::MPZ(const MPZ &op)
{
  mpz_init_set(rep, op.rep);
}

inline MPZ::MPZ(unsigned long int op)
{
  mpz_init_set_ui(rep, op);
}

inline MPZ::MPZ(unsigned int op)
{
  mpz_init_set_ui(rep, static_cast<unsigned long int>(op));
}

inline MPZ::MPZ(signed long int op)
{
  mpz_init_set_si(rep, op);
}

inline MPZ::MPZ(signed int op)
{
  mpz_init_set_si(rep, static_cast<signed long int>(op));
}

inline MPZ::MPZ(double op)
{
  mpz_init_set_d(rep, op);
}

inline MPZ::MPZ(const char *str, int base) throw(InvalidNumberException)
{
  mpz_init_set_str(rep, str, base);
}

inline mp_limb_t MPZ::getlimbn(mp_size_t n) const
{
  return mpz_getlimbn(rep, n);
}

inline unsigned long int MPZ::get_ui() const
{
  return mpz_get_ui(rep);
}

inline signed long int MPZ::get_si() const
{
  return mpz_get_si(rep);
}

inline double MPZ::get_d() const
{
  return mpz_get_d(rep);
}

inline char *MPZ::get_str(char *str, int base) const
{
  return mpz_get_str(str, base, rep);
}

inline MPZ &MPZ::operator ++()
{
  mpz_add_ui(rep, rep, 1);
  return *this;
}

inline MPZ MPZ::operator ++(int)
{
  MPZ R = *this;
  mpz_add_ui(rep, rep, 1);
  return R;
}

inline MPZ &MPZ::operator --()
{
  mpz_sub_ui(rep, rep, 1);
  return *this;
}

inline MPZ MPZ::operator --(int)
{
  MPZ R = *this;
  mpz_sub_ui(rep, rep, 1);
  return R;
}

inline MPZ &MPZ::addmul(const MPZ &op1, unsigned long int op2)
{
  mpz_addmul_ui(rep, op1.rep, op2);
  return *this;
}

inline unsigned long int MPZ::tdiv(unsigned long int d) const
{
  return mpz_tdiv_ui(rep, d);
}

inline unsigned long int MPZ::fdiv(unsigned long int d) const
{
  return mpz_fdiv_ui(rep, d);
}

inline unsigned long int MPZ::cdiv(unsigned long int d) const
{
  return mpz_cdiv_ui(rep, d);
}

inline bool MPZ::perfect_power_p() const
{
  return mpz_perfect_power_p(rep) != 0;
}

inline bool MPZ::perfect_square_p() const
{
  return mpz_perfect_square_p(rep) != 0;
}

inline int MPZ::probab_prime_p(int reps) const
{
  return mpz_probab_prime_p(rep, reps);
}

inline unsigned long int MPZ::gcd(const MPZ &op1, unsigned long int op2)
{
  return mpz_gcd_ui(0, op1.rep, op2);
}

inline unsigned long int MPZ::gcd(unsigned long int op) const
{
  return gcd(*this, op);
}

inline MPZ MPZ::invert(const MPZ &op1, const MPZ &op2) throw(NonexistentInverseException)
{
  MPZ rop;
  if (mpz_invert(rop.rep, op1.rep, op2.rep) == 0) {
    throw NonexistentInverseException();
  }
  return rop;
}

inline MPZ &MPZ::invert(const MPZ &op) throw(NonexistentInverseException)
{
  if (mpz_invert(rep, rep, op.rep) == 0) {
    throw NonexistentInverseException();
  }
  return *this;
}

inline int MPZ::jacobi(const MPZ &op1, const MPZ &op2)
{
  return mpz_jacobi(op1.rep, op2.rep);
}

inline int MPZ::jacobi(const MPZ &op) const
{
  return jacobi(*this, op);
}

inline int MPZ::legendre(const MPZ &op1, const MPZ &op2)
{
  return mpz_legendre(op1.rep, op2.rep);
}

inline int MPZ::legendre(const MPZ &op) const
{
  return legendre(*this, op);
}

inline int MPZ::kronecker(signed long int a, const MPZ &b)
{
  return mpz_si_kronecker(a, b.rep);
}

inline int MPZ::kronecker(signed int a, const MPZ &b)
{
  return kronecker(static_cast<signed long int>(a), b);
}

inline int MPZ::kronecker(unsigned long int a, const MPZ &b)
{
  return mpz_ui_kronecker(a, b.rep);
}

inline int MPZ::kronecker(unsigned int a, const MPZ &b)
{
  return kronecker(static_cast<unsigned long int>(a), b);
}

inline int MPZ::kronecker(const MPZ &a, signed long int b)
{
  return mpz_kronecker_si(a.rep, b);
}

inline int MPZ::kronecker(const MPZ &a, signed int b)
{
  return kronecker(a, static_cast<signed long int>(b));
}

inline int MPZ::kronecker(const MPZ &a, unsigned long int b)
{
  return mpz_kronecker_ui(a.rep, b);
}

inline int MPZ::kronecker(const MPZ &a, unsigned int b)
{
  return kronecker(a, static_cast<unsigned long int>(b));
}

inline int MPZ::cmp(const MPZ &op1, const MPZ &op2)
{
  return mpz_cmp(op1.rep, op2.rep);
}

inline int MPZ::cmp(const MPZ &op1, unsigned long int op2)
{
  return mpz_cmp_ui(op1.rep, op2);
}

inline int MPZ::cmp(const MPZ &op1, signed long int op2)
{
  return mpz_cmp_si(op1.rep, op2);
}

inline int MPZ::cmp(const MPZ &op) const
{
  return mpz_cmp(rep, op.rep);
}

inline int MPZ::cmp(unsigned long int op) const
{
  return mpz_cmp_ui(rep, op);
}

inline int MPZ::cmp(signed long int op) const
{
  return mpz_cmp_si(rep, op);
}

inline bool MPZ::operator ==(const MPZ &op) const
{
  return mpz_cmp(rep, op.rep) == 0;
}

inline bool MPZ::operator ==(unsigned long int op) const
{
  return mpz_cmp_ui(rep, op) == 0;
}

inline bool MPZ::operator ==(unsigned int op) const
{
  return operator ==(static_cast<unsigned long int>(op));
}

inline bool MPZ::operator ==(signed long int op) const
{
  return mpz_cmp_si(rep, op) == 0;
}

inline bool MPZ::operator ==(signed int op) const
{
  return operator ==(static_cast<signed long int>(op));
}

inline bool MPZ::operator !=(const MPZ &op) const
{
  return !operator ==(op);
}

inline bool MPZ::operator !=(unsigned long int op) const
{
  return !operator ==(op);
}

inline bool MPZ::operator !=(unsigned int op) const
{
  return operator !=(static_cast<unsigned long int>(op));
}

inline bool MPZ::operator !=(signed long int op) const
{
  return !operator ==(op);
}

inline bool MPZ::operator !=(signed int op) const
{
  return operator !=(static_cast<signed long int>(op));
}

inline bool MPZ::operator >(const MPZ &op) const
{
  return mpz_cmp(rep, op.rep) > 0;
}

inline bool MPZ::operator >(unsigned long int op) const
{
  return mpz_cmp_ui(rep, op) > 0;
}

inline bool MPZ::operator >(unsigned int op) const
{
  return operator >(static_cast<unsigned long int>(op));
}

inline bool MPZ::operator >(signed long int op) const
{
  return mpz_cmp_si(rep, op) > 0;
}

inline bool MPZ::operator >(signed int op) const
{
  return operator >(static_cast<signed long int>(op));
}

inline bool MPZ::operator >=(const MPZ &op) const
{
  return mpz_cmp(rep, op.rep) >= 0;
}

inline bool MPZ::operator >=(unsigned long int op) const
{
  return mpz_cmp_ui(rep, op) >= 0;
}

inline bool MPZ::operator >=(unsigned int op) const
{
  return operator >=(static_cast<unsigned long int>(op));
}

inline bool MPZ::operator >=(signed long int op) const
{
  return mpz_cmp_si(rep, op) >= 0;
}

inline bool MPZ::operator >=(signed int op) const
{
  return operator >=(static_cast<signed long int>(op));
}

inline bool MPZ::operator <=(const MPZ &op) const
{
  return mpz_cmp(rep, op.rep) <= 0;
}

inline bool MPZ::operator <=(unsigned long int op) const
{
  return mpz_cmp_ui(rep, op) <= 0;
}

inline bool MPZ::operator <=(unsigned int op) const
{
  return operator <=(static_cast<unsigned long int>(op));
}

inline bool MPZ::operator <=(signed long int op) const
{
  return mpz_cmp_si(rep, op) <= 0;
}

inline bool MPZ::operator <=(signed int op) const
{
  return operator <=(static_cast<signed long int>(op));
}

inline bool MPZ::operator <(const MPZ &op) const
{
  return mpz_cmp(rep, op.rep) < 0;
}

inline bool MPZ::operator <(unsigned long int op) const
{
  return mpz_cmp_ui(rep, op) < 0;
}

inline bool MPZ::operator <(unsigned int op) const
{
  return operator <(static_cast<unsigned long int>(op));
}

inline bool MPZ::operator <(signed long int op) const
{
  return mpz_cmp_si(rep, op) < 0;
}

inline bool MPZ::operator <(signed int op) const
{
  return operator <(static_cast<signed long int>(op));
}

inline int MPZ::cmpabs(const MPZ &op1, const MPZ &op2)
{
  return mpz_cmpabs(op1.rep, op2.rep);
}

inline int MPZ::cmpabs(const MPZ &op1, unsigned long int op2)
{
  return mpz_cmpabs_ui(op1.rep, op2);
}

inline int MPZ::cmpabs(const MPZ &op) const
{
  return mpz_cmpabs(rep, op.rep);
}

inline int MPZ::cmpabs(unsigned long int op) const
{
  return mpz_cmpabs_ui(rep, op);
}

inline int MPZ::sgn() const
{
  return mpz_sgn(rep);
}

inline int MPZ::popcount() const
{
  return mpz_popcount(rep);
}

inline int MPZ::hamdist(const MPZ &op) const
{
  return mpz_hamdist(rep, op.rep);
}

inline int MPZ::scan0(unsigned long int starting_bit) const
{
  return mpz_scan0(rep, starting_bit);
}

inline int MPZ::scan1(unsigned long int starting_bit) const
{
  return mpz_scan1(rep, starting_bit);
}

inline MPZ &MPZ::setbit(unsigned long int index)
{
  mpz_setbit(rep, index);
  return *this;
}

inline MPZ &MPZ::clrbit(unsigned long int index)
{
  mpz_clrbit(rep, index);
  return *this;
}

inline int MPZ::tstbit(unsigned long int index) const
{
  return mpz_tstbit(rep, index);
}

#ifdef _GMP_H_HAVE_FILE

inline size_t MPZ::out_str(FILE *stream, int base) const
{
  return mpz_out_str(stream, base, rep);
}

inline MPZ MPZ::inp_str(FILE *stream, int base) throw(IntegerInputException)
{
  MPZ rop;
  if (mpz_inp_str(rop.rep, stream, base) == 0) {
    throw IntegerInputException();
  }
  return rop;
}

inline size_t MPZ::out_raw(FILE *stream) const
{
  return mpz_out_raw(stream, rep);
}

inline MPZ MPZ::inp_raw(FILE *stream) throw(IntegerInputException)
{
  MPZ rop;
  if (mpz_inp_raw(rop.rep, stream) == 0) {
    throw IntegerInputException();
  }
  return rop;
}

#endif // GMP_H_HAVE_FILE

inline bool MPZ::fits_ulong_p() const
{
  return mpz_fits_ulong_p(rep) != 0;
}

inline bool MPZ::fits_slong_p() const
{
  return mpz_fits_slong_p(rep) != 0;
}

inline bool MPZ::fits_uint_p() const
{
  return mpz_fits_uint_p(rep) != 0;
}

inline bool MPZ::fits_sint_p() const
{
  return mpz_fits_sint_p(rep) != 0;
}

inline bool MPZ::fits_ushort_p() const
{
  return mpz_fits_ushort_p(rep) != 0;
}

inline bool MPZ::fits_sshort_p() const
{
  return mpz_fits_sshort_p(rep) != 0;
}

inline bool MPZ::odd() const
{
  return mpz_odd_p(rep) != 0;
}

inline bool MPZ::even() const
{
  return mpz_even_p(rep) != 0;
}

inline size_t MPZ::size() const
{
  return mpz_size(rep);
}

inline size_t MPZ::sizeinbase(int base) const
{
  return mpz_sizeinbase(rep, base);
}

#ifdef _GMPCPP_H_HAVE_IOSTREAM

inline std::ostream &operator <<(std::ostream &s, const MPZ &op)
{
  char *str = op.get_str();
  s << str;
  free(str);
  return s;
}

inline std::istream &operator >>(std::istream &s, MPZ &op)
{
  int size = 1, len = 0;
  char *str = static_cast<char *>(malloc(size));
  while (true) {
    char c = 0;
    s >> c;
    if (c >= '0' && c <= '9' || c >= 'a' && c <= 'z' || c >= 'A' && c <= 'Z') {
      // TODO: check for valid char in base?
      str[len++] = c;
      if (len >= size) {
        size <<= 1;
        str = static_cast<char *>(realloc(str, size));
      }
    } else {
      s.putback(c);
      break;
    }
  }
  str[len] = 0;
  op.set(str);
  free(str);
  return s;
}

#endif // _GMPCPP_H_HAVE_IOSTREAM

/*
 * class RANDSTATE methods
 */

inline RANDSTATE::RANDSTATE(gmp_randalg_t randalg, unsigned long int size)
{
  gmp_randinit(state, randalg, size);
}

inline RANDSTATE::RANDSTATE(const MPZ &a, unsigned long int c, unsigned long int m2exp)
{
  gmp_randinit_lc_2exp(state, const_cast<__mpz_struct *>(a.rep), c, m2exp);
}

inline RANDSTATE::~RANDSTATE()
{
  gmp_randclear(state);
}

inline void RANDSTATE::randseed(const MPZ &seed)
{
  gmp_randseed(state, const_cast<__mpz_struct *>(seed.rep));
}

inline void RANDSTATE::randseed(unsigned long int seed)
{
  gmp_randseed_ui(state, seed);
}

inline MPZ RANDSTATE::urandomb(unsigned long int n)
{
  MPZ rop;
  mpz_urandomb(rop.rep, state, n);
  return rop;
}

inline MPZ RANDSTATE::urandomm(const MPZ &n)
{
  MPZ rop;
  mpz_urandomm(rop.rep, state, const_cast<__mpz_struct *>(n.rep));
  return rop;
}

inline MPZ RANDSTATE::rrandomb(unsigned long int n)
{
  MPZ rop;
  mpz_rrandomb(rop.rep, state, n);
  return rop;
}

/*
 * class MPQ methods
 */

inline MPQ::MPQ()
{
  mpq_init(rep);
}

inline MPQ::MPQ(const MPQ &op)
{
  mpq_init(rep);
  mpq_set(rep, op.rep);
}

inline MPQ::~MPQ()
{
  mpq_clear(rep);
}

inline MPQ::operator const __mpq_struct *() const
{
  return rep;
}

inline MPQ::MPQ(unsigned long int op1, unsigned long int op2)
{
  mpq_init(rep);
  mpq_set_ui(rep, op1, op2);
}

inline MPQ::MPQ(signed long int op1, unsigned long int op2)
{
  mpq_init(rep);
  mpq_set_si(rep, op1, op2);
}

inline MPQ &MPQ::set(const MPQ &op)
{
  if (&op == this) {
    return *this;
  }
  mpq_set(rep, op.rep);
  return *this;
}

inline MPQ &MPQ::set(const MPZ &op)
{
  mpq_set_z(rep, op.rep);
  return *this;
}

inline MPQ &MPQ::set(unsigned long int op1, unsigned long int op2)
{
  mpq_set_ui(rep, op1, op2);
  return *this;
}

inline MPQ &MPQ::set(signed long int op1, unsigned long int op2)
{
  mpq_set_si(rep, op1, op2);
  return *this;
}

inline MPQ &MPQ::operator =(const MPQ &op)
{
  return set(op);
}

inline MPQ &MPQ::operator =(const MPZ &op)
{
  return set(op);
}

inline MPQ &MPQ::swap(MPQ &rop2)
{
  mpq_swap(rep, rop2.rep);
  return *this;
}

inline int MPQ::cmp(const MPQ &op1, const MPQ &op2)
{
  return mpq_cmp(op1.rep, op2.rep);
}

inline int MPQ::cmp(const MPQ &op) const
{
  return mpq_cmp(rep, op.rep);
}

inline bool MPQ::operator ==(const MPQ &op) const
{
  return mpq_equal(rep, op.rep) == 0;
}

inline bool MPQ::operator !=(const MPQ &op) const
{
  return !operator ==(op);
}

inline bool MPQ::operator >(const MPQ &op) const
{
  return mpq_cmp(rep, op.rep) > 0;
}

inline bool MPQ::operator >=(const MPQ &op) const
{
  return mpq_cmp(rep, op.rep) >= 0;
}

inline bool MPQ::operator <=(const MPQ &op) const
{
  return mpq_cmp(rep, op.rep) <= 0;
}

inline bool MPQ::operator <(const MPQ &op) const
{
  return mpq_cmp(rep, op.rep) < 0;
}

inline int MPQ::sgn() const
{
  return mpq_sgn(rep);
}

#ifdef _GMP_H_HAVE_FILE

inline size_t MPQ::out_str(FILE *stream, int base) const
{
  return mpq_out_str(stream, base, rep);
}

#endif // GMP_H_HAVE_FILE

inline double MPQ::get_d() const
{
  return mpq_get_d(rep);
}

inline void MPQ::set_d(double d)
{
  mpq_set_d(rep, d);
}

inline void MPQ::set_num(const MPZ &numerator)
{
  mpq_set_num(rep, numerator);
}

inline void MPQ::set_den(const MPZ &denominator)
{
  mpq_set_den(rep, denominator);
}

inline MPZ MPQ::get_num() const
{
  MPZ rop;
  mpq_get_num(rop.rep, rep);
  return rop;
}

inline MPZ MPQ::get_den() const
{
  MPZ rop;
  mpq_get_den(rop.rep, rep);
  return rop;
}

/*
 * class MPF methods
 */

inline void MPF::set_default_prec(unsigned long int prec)
{
  mpf_set_default_prec(prec);
}

inline MPF::MPF()
{
  mpf_init(rep);
}

inline MPF::MPF(bool, unsigned long int prec)
{
  mpf_init2(rep, prec);
}

inline MPF::~MPF()
{
  mpf_clear(rep);
}

inline void MPF::set_prec(unsigned long int prec)
{
  mpf_set_prec(rep, prec);
}

inline unsigned long int MPF::get_prec() const
{
  return mpf_get_prec(rep);
}

inline void MPF::set_prec_raw(unsigned long int prec)
{
  mpf_set_prec_raw(rep, prec);
}

inline MPF::operator const __mpf_struct *() const
{
  return rep;
}

inline MPF &MPF::set(const MPF &op)
{
  if (&op == this) {
    return *this;
  }
  mpf_set(rep, op.rep);
  return *this;
}

inline MPF &MPF::set(unsigned long int op)
{
  mpf_set_ui(rep, op);
  return *this;
}

inline MPF &MPF::set(unsigned int op)
{
  return set(static_cast<unsigned long int>(op));
}

inline MPF &MPF::set(signed long int op)
{
  mpf_set_si(rep, op);
  return *this;
}

inline MPF &MPF::set(signed int op)
{
  return set(static_cast<signed long int>(op));
}

inline MPF &MPF::set(double op)
{
  mpf_set_d(rep, op);
  return *this;
}

inline MPF &MPF::set(const MPZ &op)
{
  mpf_set_z(rep, op.rep);
  return *this;
}

inline MPF &MPF::set(const MPQ &op)
{
  mpf_set_q(rep, op.rep);
  return *this;
}

inline MPF &MPF::set(const char *str, int base) throw(InvalidNumberException)
{
  if (mpf_set_str(rep, str, base) != 0) {
    throw InvalidNumberException();
  }
  return *this;
}

inline MPF &MPF::operator =(const MPF &op)
{
  return set(op);
}

inline MPF &MPF::operator =(unsigned long int op)
{
  return set(op);
}

inline MPF &MPF::operator =(unsigned int op)
{
  return set(op);
}

inline MPF &MPF::operator =(signed long int op)
{
  return set(op);
}

inline MPF &MPF::operator =(signed int op)
{
  return set(op);
}

inline MPF &MPF::operator =(double op)
{
  return set(op);
}

inline MPF &MPF::operator =(const MPZ &op)
{
  return set(op);
}

inline MPF &MPF::operator =(const MPQ &op)
{
  return set(op);
}

inline MPF &MPF::operator =(const char *str) throw (InvalidNumberException)
{
  return set(str);
}

inline MPF &MPF::swap(MPF &rop2)
{
  mpf_swap(rep, rop2.rep);
  return *this;
}

inline MPF::MPF(const MPF &op)
{
  mpf_init_set(rep, op.rep);
}

inline MPF::MPF(unsigned long int op)
{
  mpf_init_set_ui(rep, op);
}

inline MPF::MPF(unsigned int op)
{
  mpf_init_set_ui(rep, static_cast<unsigned long int>(op));
}

inline MPF::MPF(signed long int op)
{
  mpf_init_set_si(rep, op);
}

inline MPF::MPF(signed int op)
{
  mpf_init_set_si(rep, static_cast<signed long int>(op));
}

inline MPF::MPF(double op)
{
  mpf_init_set_d(rep, op);
}

inline MPF::MPF(const char *str, int base) throw(InvalidNumberException)
{
  mpf_init_set_str(rep, str, base);
}

inline double MPF::get_d() const
{
  return mpf_get_d(rep);
}

inline char *MPF::get_str(char *str, mp_exp_t *expptr, int base, size_t ndigits) const
{
  return mpf_get_str(str, expptr, base, ndigits, rep);
}

inline MPF &MPF::operator ++()
{
  mpf_add_ui(rep, rep, 1);
  return *this;
}

inline MPF MPF::operator ++(int)
{
  MPF R = *this;
  mpf_add_ui(rep, rep, 1);
  return R;
}

inline MPF &MPF::operator --()
{
  mpf_sub_ui(rep, rep, 1);
  return *this;
}

inline MPF MPF::operator --(int)
{
  MPF R = *this;
  mpf_sub_ui(rep, rep, 1);
  return R;
}

inline int MPF::cmp(const MPF &op1, const MPF &op2)
{
  return mpf_cmp(op1.rep, op2.rep);
}

inline int MPF::cmp(const MPF &op1, unsigned long int op2)
{
  return mpf_cmp_ui(op1.rep, op2);
}

inline int MPF::cmp(const MPF &op1, signed long int op2)
{
  return mpf_cmp_si(op1.rep, op2);
}

inline int MPF::cmp(const MPF &op) const
{
  return mpf_cmp(rep, op.rep);
}

inline int MPF::cmp(unsigned long int op) const
{
  return mpf_cmp_ui(rep, op);
}

inline int MPF::cmp(signed long int op) const
{
  return mpf_cmp_si(rep, op);
}

inline bool MPF::operator ==(const MPF &op) const
{
  return mpf_cmp(rep, op.rep) == 0;
}

inline bool MPF::operator ==(unsigned long int op) const
{
  return mpf_cmp_ui(rep, op) == 0;
}

inline bool MPF::operator ==(unsigned int op) const
{
  return operator ==(static_cast<unsigned long int>(op));
}

inline bool MPF::operator ==(signed long int op) const
{
  return mpf_cmp_si(rep, op) == 0;
}

inline bool MPF::operator ==(signed int op) const
{
  return operator ==(static_cast<signed long int>(op));
}

inline bool MPF::operator !=(const MPF &op) const
{
  return !operator ==(op);
}

inline bool MPF::operator !=(unsigned long int op) const
{
  return !operator ==(op);
}

inline bool MPF::operator !=(unsigned int op) const
{
  return operator !=(static_cast<unsigned long int>(op));
}

inline bool MPF::operator !=(signed long int op) const
{
  return !operator ==(op);
}

inline bool MPF::operator !=(signed int op) const
{
  return operator !=(static_cast<signed long int>(op));
}

inline bool MPF::operator >(const MPF &op) const
{
  return mpf_cmp(rep, op.rep) > 0;
}

inline bool MPF::operator >(unsigned long int op) const
{
  return mpf_cmp_ui(rep, op) > 0;
}

inline bool MPF::operator >(unsigned int op) const
{
  return operator >(static_cast<unsigned long int>(op));
}

inline bool MPF::operator >(signed long int op) const
{
  return mpf_cmp_si(rep, op) > 0;
}

inline bool MPF::operator >(signed int op) const
{
  return operator >(static_cast<signed long int>(op));
}

inline bool MPF::operator >=(const MPF &op) const
{
  return mpf_cmp(rep, op.rep) >= 0;
}

inline bool MPF::operator >=(unsigned long int op) const
{
  return mpf_cmp_ui(rep, op) >= 0;
}

inline bool MPF::operator >=(unsigned int op) const
{
  return operator >=(static_cast<unsigned long int>(op));
}

inline bool MPF::operator >=(signed long int op) const
{
  return mpf_cmp_si(rep, op) >= 0;
}

inline bool MPF::operator >=(signed int op) const
{
  return operator >=(static_cast<signed long int>(op));
}

inline bool MPF::operator <=(const MPF &op) const
{
  return mpf_cmp(rep, op.rep) <= 0;
}

inline bool MPF::operator <=(unsigned long int op) const
{
  return mpf_cmp_ui(rep, op) <= 0;
}

inline bool MPF::operator <=(unsigned int op) const
{
  return operator <=(static_cast<unsigned long int>(op));
}

inline bool MPF::operator <=(signed long int op) const
{
  return mpf_cmp_si(rep, op) <= 0;
}

inline bool MPF::operator <=(signed int op) const
{
  return operator <=(static_cast<signed long int>(op));
}

inline bool MPF::operator <(const MPF &op) const
{
  return mpf_cmp(rep, op.rep) < 0;
}

inline bool MPF::operator <(unsigned long int op) const
{
  return mpf_cmp_ui(rep, op) < 0;
}

inline bool MPF::operator <(unsigned int op) const
{
  return operator <(static_cast<unsigned long int>(op));
}

inline bool MPF::operator <(signed long int op) const
{
  return mpf_cmp_si(rep, op) < 0;
}

inline bool MPF::operator <(signed int op) const
{
  return operator <(static_cast<signed long int>(op));
}

inline int MPF::eq(const MPF &op1, const MPF &op2, unsigned long int op3)
{
  return mpf_eq(op1.rep, op2.rep, op3);
}

inline int MPF::eq(const MPF &op2, unsigned long int op3) const
{
  return mpf_eq(rep, op2.rep, op3);
}

inline int MPF::sgn() const
{
  return mpf_sgn(rep);
}

#ifdef _GMP_H_HAVE_FILE

inline size_t MPF::out_str(FILE *stream, int base, size_t n_digits) const
{
  return mpf_out_str(stream, base, n_digits, rep);
}

inline MPF MPF::inp_str(FILE *stream, int base) throw(IntegerInputException)
{
  MPF rop;
  if (mpf_inp_str(rop.rep, stream, base) == 0) {
    throw IntegerInputException();
  }
  return rop;
}

#endif // GMP_H_HAVE_FILE
