#include <stdio.h>
#include <math.h>
#include <iostream>
#include <fstream>
#include <string>

#include "gmpcpp.h"

using namespace GMP;

const char *c0 = "299792458";
const char *M727 = "706003489677054374237277210551156965837838477962894381170850482715673"
                   "457590299624976468480248807499242724466374570999144530824216469597736"
                   "906638272121736526607699022870679030143158018123175881930939339869708"
                   "632591433727";


int failures = 0;

string str(const mpz_t op)
{
  char *p = mpz_get_str(NULL, 10, op);
  string r = p;
  free(p);
  return r;
}

void fail(const char *s)
{
  printf("%s\n", s);
  failures++;
}

namespace test_MPZ {

void set_MPZ()
{
  mpz_t a, b;
  mpz_init(a);
  mpz_init(b);
  mpz_set_ui(a, 3);
  mpz_set(b, a);
  MPZ A, B;
  A = 3;
  B.set(A);
  if (str(B) != str(b)) fail("set(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void set_unsigned_long_int()
{
  mpz_t a;
  mpz_init(a);
  mpz_set_ui(a, ULONG_MAX);
  MPZ A;
  A.set(ULONG_MAX);
  if (str(A) != str(a)) fail("set(unsigned long int)");
  mpz_clear(a);
}

void set_unsigned_int()
{
  mpz_t a;
  mpz_init(a);
  mpz_set_ui(a, UINT_MAX);
  MPZ A;
  A.set(UINT_MAX);
  if (str(A) != str(a)) fail("set(unsigned int)");
  mpz_clear(a);
}

void set_signed_long_int()
{
  mpz_t a;
  mpz_init(a);
  mpz_set_si(a, LONG_MIN);
  MPZ A;
  A.set(LONG_MIN);
  if (str(A) != str(a)) fail("set(signed long int)");
  mpz_clear(a);
}

void set_signed_int()
{
  mpz_t a;
  mpz_init(a);
  mpz_set_si(a, INT_MIN);
  MPZ A;
  A.set(INT_MIN);
  if (str(A) != str(a)) fail("set(signed int)");
  mpz_clear(a);
}

void set_double()
{
  mpz_t a;
  mpz_init(a);
  mpz_set_d(a, M_E);
  MPZ A;
  A.set(M_E);
  if (str(A) != str(a)) fail("set(double)");
  mpz_clear(a);
}

void set_char()
{
  mpz_t a;
  mpz_init(a);
  mpz_set_str(a, c0, 10);
  MPZ A;
  A.set(c0);
  if (str(A) != str(a)) fail("set(const char *)");
  mpz_clear(a);
}

void operator_eq_MPZ()
{
  mpz_t a, b;
  mpz_init(a);
  mpz_init(b);
  mpz_set_ui(a, 3);
  mpz_set(b, a);
  MPZ A, B;
  A = 3;
  B = A;
  if (str(B) != str(b)) fail("operator =(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void operator_eq_unsigned_long_int()
{
  mpz_t a;
  mpz_init(a);
  mpz_set_ui(a, ULONG_MAX);
  MPZ A;
  A = ULONG_MAX;
  if (str(A) != str(a)) fail("operator =(unsigned long int)");
  mpz_clear(a);
}

void operator_eq_unsigned_int()
{
  mpz_t a;
  mpz_init(a);
  mpz_set_ui(a, UINT_MAX);
  MPZ A;
  A = UINT_MAX;
  if (str(A) != str(a)) fail("operator =(unsigned int)");
  mpz_clear(a);
}

void operator_eq_signed_long_int()
{
  mpz_t a;
  mpz_init(a);
  mpz_set_si(a, LONG_MIN);
  MPZ A;
  A = LONG_MIN;
  if (str(A) != str(a)) fail("operator =(signed long int)");
  mpz_clear(a);
}

void operator_eq_signed_int()
{
  mpz_t a;
  mpz_init(a);
  mpz_set_si(a, INT_MIN);
  MPZ A;
  A = INT_MIN;
  if (str(A) != str(a)) fail("operator =(signed int)");
  mpz_clear(a);
}

void operator_eq_double()
{
  mpz_t a;
  mpz_init(a);
  mpz_set_d(a, M_E);
  MPZ A;
  A = M_E;
  if (str(A) != str(a)) fail("operator =(double)");
  mpz_clear(a);
}

void operator_eq_char()
{
  mpz_t a;
  mpz_init(a);
  mpz_set_str(a, c0, 10);
  MPZ A;
  A = c0;
  if (str(A) != str(a)) fail("operator =(const char *)");
  mpz_clear(a);
}

void swap()
{
  mpz_t a, b;
  mpz_init(a);
  mpz_init(b);
  mpz_set_ui(a, ULONG_MAX);
  mpz_set_si(b, LONG_MIN);
  mpz_swap(a, b);
  MPZ A, B;
  A = ULONG_MAX;
  B = LONG_MIN;
  A.swap(B);
  if (str(A) != str(a)) fail("swap()");
  if (str(B) != str(b)) fail("swap()");
  mpz_clear(a);
  mpz_clear(b);
}

void constructor_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 3);
  mpz_init_set(b, a);
  MPZ A = 3;
  MPZ B(A);
  if (str(B) != str(b)) fail("MPZ(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void constructor_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, ULONG_MAX);
  MPZ A(ULONG_MAX);
  if (str(A) != str(a)) fail("MPZ(unsigned long int)");
  mpz_clear(a);
}

void constructor_unsigned_int()
{
  mpz_t a;
  mpz_init_set_ui(a, UINT_MAX);
  MPZ A(UINT_MAX);
  if (str(A) != str(a)) fail("MPZ(unsigned int)");
  mpz_clear(a);
}

void constructor_signed_long_int()
{
  mpz_t a;
  mpz_init_set_si(a, LONG_MIN);
  MPZ A(LONG_MIN);
  if (str(A) != str(a)) fail("MPZ(signed long int)");
  mpz_clear(a);
}

void constructor_signed_int()
{
  mpz_t a;
  mpz_init_set_si(a, INT_MIN);
  MPZ A(INT_MIN);
  if (str(A) != str(a)) fail("MPZ(signed int)");
  mpz_clear(a);
}

void constructor_double()
{
  mpz_t a;
  mpz_init_set_d(a, M_E);
  MPZ A(M_E);
  if (str(A) != str(a)) fail("MPZ(double)");
  mpz_clear(a);
}

void constructor_char()
{
  mpz_t a;
  mpz_init_set_str(a, c0, 10);
  MPZ A(c0);
  if (str(A) != str(a)) fail("MPZ(const char *)");
  mpz_clear(a);
}

void getlimbn()
{
  mpz_t a;
  mpz_init_set_str(a, M727, 10);
  MPZ A = M727;
  for (size_t i = 0; i < A.size(); i++) {
    if (A.getlimbn(i) != mpz_getlimbn(a, i)) fail("getlimbn()");
  }
  mpz_clear(a);
}

void get_ui()
{
  mpz_t a;
  mpz_init_set_ui(a, ULONG_MAX);
  MPZ A = ULONG_MAX;
  if (A.get_ui() != mpz_get_ui(a)) fail("get_ui()");
  mpz_clear(a);
}

void get_si()
{
  mpz_t a;
  mpz_init_set_si(a, LONG_MIN);
  MPZ A = LONG_MIN;
  if (A.get_si() != mpz_get_si(a)) fail("get_si()");
  mpz_clear(a);
}

void get_d()
{
  mpz_t a;
  mpz_init_set_d(a, M_E);
  MPZ A = M_E;
  if (A.get_d() != mpz_get_d(a)) fail("get_d()");
  mpz_clear(a);
}

void get_str()
{
  mpz_t a;
  mpz_init_set_str(a, c0, 10);
  MPZ A = c0;
  char *s = mpz_get_str(NULL, 10, a);
  if (A.get_str() != string(s)) fail("get_str()");
  free(s);
  mpz_clear(a);
}

void add_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 12345);
  mpz_init_set_ui(b, 77777);
  mpz_init(r);
  mpz_add(r, a, b);
  MPZ A = 12345, B = 77777;
  MPZ R = MPZ::add(A, B);
  if (str(R) != str(r)) fail("add(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void add_MPZ_unsigned_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_add_ui(r, a, 77777);
  MPZ A = 12345;
  MPZ R = MPZ::add(A, 77777);
  if (str(R) != str(r)) fail("add(const MPZ &, unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void operator_plus_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 12345);
  mpz_init_set_ui(b, 77777);
  mpz_init(r);
  mpz_add(r, a, b);
  MPZ A = 12345, B = 77777;
  MPZ R = A + B;
  if (str(R) != str(r)) fail("operator +(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void operator_plus_unsigned_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_add_ui(r, a, 77777);
  MPZ A = 12345;
  MPZ R = A + 77777;
  if (str(R) != str(r)) fail("operator +(unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void add_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 12345);
  mpz_init_set_ui(b, 77777);
  mpz_add(a, a, b);
  MPZ A = 12345, B = 77777;
  A.add(B);
  if (str(A) != str(a)) fail("add(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void add_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_add_ui(a, a, 77777);
  MPZ A = 12345;
  A.add(77777);
  if (str(A) != str(a)) fail("add(unsigned long int)");
  mpz_clear(a);
}

void operator_plusequals_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 12345);
  mpz_init_set_ui(b, 77777);
  mpz_add(a, a, b);
  MPZ A = 12345, B = 77777;
  A += B;
  if (str(A) != str(a)) fail("operator +=(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void operator_plusequals_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_add_ui(a, a, 77777);
  MPZ A = 12345;
  A += 77777;
  if (str(A) != str(a)) fail("operator +=(unsigned long int)");
  mpz_clear(a);
}

void operator_increment_prefix()
{
  MPZ A = 98765;
  MPZ B = A;
  MPZ C = ++B;
  if (B == A) fail("operator ++()");
  if (C != B) fail("operator ++()");
}

void operator_increment_postfix()
{
  MPZ A = 98765;
  MPZ B = A;
  MPZ C = B++;
  if (B == A) fail("operator ++(int)");
  if (C == B) fail("operator ++(int)");
}

void sub_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 12345);
  mpz_init_set_ui(b, 77777);
  mpz_init(r);
  mpz_sub(r, a, b);
  MPZ A = 12345, B = 77777;
  MPZ R = MPZ::sub(A, B);
  if (str(R) != str(r)) fail("sub(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void sub_MPZ_unsigned_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_sub_ui(r, a, 77777);
  MPZ A = 12345;
  MPZ R = MPZ::sub(A, 77777);
  if (str(R) != str(r)) fail("sub(const MPZ &, unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void operator_minus_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 12345);
  mpz_init_set_ui(b, 77777);
  mpz_init(r);
  mpz_sub(r, a, b);
  MPZ A = 12345, B = 77777;
  MPZ R = A - B;
  if (str(R) != str(r)) fail("operator -(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void operator_minus_unsigned_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_sub_ui(r, a, 77777);
  MPZ A = 12345;
  MPZ R = A - 77777;
  if (str(R) != str(r)) fail("operator -(unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void sub_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 12345);
  mpz_init_set_ui(b, 77777);
  mpz_sub(a, a, b);
  MPZ A = 12345, B = 77777;
  A.sub(B);
  if (str(A) != str(a)) fail("sub(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void sub_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_sub_ui(a, a, 77777);
  MPZ A = 12345;
  A.sub(77777);
  if (str(A) != str(a)) fail("sub(unsigned long int)");
  mpz_clear(a);
}

void operator_minusequals_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 12345);
  mpz_init_set_ui(b, 77777);
  mpz_sub(a, a, b);
  MPZ A = 12345, B = 77777;
  A -= B;
  if (str(A) != str(a)) fail("operator -=(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void operator_minusequals_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_sub_ui(a, a, 77777);
  MPZ A = 12345;
  A -= 77777;
  if (str(A) != str(a)) fail("operator -=(unsigned long int)");
  mpz_clear(a);
}

void operator_decrement_prefix()
{
  MPZ A = 98765;
  MPZ B = A;
  MPZ C = --B;
  if (B == A) fail("operator --()");
  if (C != B) fail("operator --()");
}

void operator_decrement_postfix()
{
  MPZ A = 98765;
  MPZ B = A;
  MPZ C = B--;
  if (B == A) fail("operator --(int)");
  if (C == B) fail("operator --(int)");
}

void mul_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 12345);
  mpz_init_set_ui(b, 77777);
  mpz_init(r);
  mpz_mul(r, a, b);
  MPZ A = 12345, B = 77777;
  MPZ R = MPZ::mul(A, B);
  if (str(R) != str(r)) fail("mul(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void mul_MPZ_unsigned_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_mul_ui(r, a, ULONG_MAX);
  MPZ A = 12345;
  MPZ R = MPZ::mul(A, ULONG_MAX);
  if (str(R) != str(r)) fail("mul(const MPZ &, unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void mul_MPZ_unsigned_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_mul_ui(r, a, UINT_MAX);
  MPZ A = 12345;
  MPZ R = MPZ::mul(A, UINT_MAX);
  if (str(R) != str(r)) fail("mul(const MPZ &, unsigned int)");
  mpz_clear(a);
  mpz_clear(r);
}

void mul_MPZ_signed_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_mul_si(r, a, LONG_MIN);
  MPZ A = 12345;
  MPZ R = MPZ::mul(A, LONG_MIN);
  if (str(R) != str(r)) fail("mul(const MPZ &, signed long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void mul_MPZ_signed_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_mul_si(r, a, INT_MIN);
  MPZ A = 12345;
  MPZ R = MPZ::mul(A, INT_MIN);
  if (str(R) != str(r)) fail("mul(const MPZ &, signed int)");
  mpz_clear(a);
  mpz_clear(r);
}

void operator_times_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 12345);
  mpz_init_set_ui(b, 77777);
  mpz_init(r);
  mpz_mul(r, a, b);
  MPZ A = 12345, B = 77777;
  MPZ R = A * B;
  if (str(R) != str(r)) fail("operator *(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void operator_times_unsigned_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_mul_ui(r, a, ULONG_MAX);
  MPZ A = 12345;
  MPZ R = A * ULONG_MAX;
  if (str(R) != str(r)) fail("operator *(unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void operator_times_unsigned_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_mul_ui(r, a, UINT_MAX);
  MPZ A = 12345;
  MPZ R = A * UINT_MAX;
  if (str(R) != str(r)) fail("operator *(unsigned int)");
  mpz_clear(a);
  mpz_clear(r);
}

void operator_times_signed_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_mul_si(r, a, LONG_MIN);
  MPZ A = 12345;
  MPZ R = A * LONG_MIN;
  if (str(R) != str(r)) fail("operator *(signed long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void operator_times_signed_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_mul_si(r, a, INT_MIN);
  MPZ A = 12345;
  MPZ R = A * INT_MIN;
  if (str(R) != str(r)) fail("operator *(signed int)");
  mpz_clear(a);
  mpz_clear(r);
}

void mul_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 12345);
  mpz_init_set_ui(b, 77777);
  mpz_mul(a, a, b);
  MPZ A = 12345, B = 77777;
  A.mul(B);
  if (str(A) != str(a)) fail("mul(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void mul_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_mul_ui(a, a, ULONG_MAX);
  MPZ A = 12345;
  A.mul(ULONG_MAX);
  if (str(A) != str(a)) fail("mul(unsigned long int)");
  mpz_clear(a);
}

void mul_unsigned_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_mul_ui(a, a, INT_MAX);
  MPZ A = 12345;
  A.mul(INT_MAX);
  if (str(A) != str(a)) fail("mul(unsigned int)");
  mpz_clear(a);
}

void mul_signed_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_mul_si(a, a, LONG_MIN);
  MPZ A = 12345;
  A.mul(LONG_MIN);
  if (str(A) != str(a)) fail("mul(signed long int)");
  mpz_clear(a);
}

void mul_signed_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_mul_si(a, a, INT_MIN);
  MPZ A = 12345;
  A.mul(INT_MIN);
  if (str(A) != str(a)) fail("mul(signed int)");
  mpz_clear(a);
}

void operator_timesequals_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 12345);
  mpz_init_set_ui(b, 77777);
  mpz_mul(a, a, b);
  MPZ A = 12345, B = 77777;
  A *= B;
  if (str(A) != str(a)) fail("operator *=(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void operator_timesequals_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_mul_ui(a, a, ULONG_MAX);
  MPZ A = 12345;
  A *= ULONG_MAX;
  if (str(A) != str(a)) fail("operator *=(unsigned long int)");
  mpz_clear(a);
}

void operator_timesequals_unsigned_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_mul_ui(a, a, UINT_MAX);
  MPZ A = 12345;
  A *= UINT_MAX;
  if (str(A) != str(a)) fail("operator *=(unsigned int)");
  mpz_clear(a);
}

void operator_timesequals_signed_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_mul_si(a, a, LONG_MIN);
  MPZ A = 12345;
  A *= LONG_MIN;
  if (str(A) != str(a)) fail("operator *=(signed long int)");
  mpz_clear(a);
}

void operator_timesequals_signed_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_mul_si(a, a, INT_MIN);
  MPZ A = 12345;
  A *= INT_MIN;
  if (str(A) != str(a)) fail("operator *=(signed int)");
  mpz_clear(a);
}

void addmul()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init_set_ui(r, 5);
  mpz_addmul_ui(r, a, 56789);
  MPZ A = 12345;
  MPZ R = 5;
  R.addmul(A, 56789);
  if (str(R) != str(r)) fail("addmul(const MPZ &, unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void mul_2exp_MPZ_unsigned_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_mul_2exp(r, a, 56789);
  MPZ A = 12345;
  MPZ R = MPZ::mul_2exp(A, 56789);
  if (str(R) != str(r)) fail("mul_2exp(const MPZ &, unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void operator_shl_unsigned_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_mul_2exp(r, a, 56789);
  MPZ A = 12345;
  MPZ R = A << 56789;
  if (str(R) != str(r)) fail("operator <<(unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void mul_2exp_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_mul_2exp(a, a, 56789);
  MPZ A = 12345;
  A.mul_2exp(56789);
  if (str(A) != str(a)) fail("mul_2exp(unsigned long int)");
  mpz_clear(a);
}

void operator_shlequals_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_mul_2exp(a, a, 56789);
  MPZ A = 12345;
  A <<= 56789;
  if (str(A) != str(a)) fail("operator <<=(unsigned long int)");
  mpz_clear(a);
}

void neg_MPZ()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_neg(r, a);
  MPZ A = 12345;
  MPZ R = MPZ::neg(A);
  if (str(R) != str(r)) fail("neg(const MPZ &)");
  mpz_clear(a);
  mpz_clear(r);
}

void neg()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_neg(a, a);
  MPZ A = 12345;
  A.neg();
  if (str(A) != str(a)) fail("neg()");
  mpz_clear(a);
}

void operator_minus()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_neg(r, a);
  MPZ A = 12345;
  MPZ R = -A;
  if (str(R) != str(r)) fail("operator -()");
  mpz_clear(a);
  mpz_clear(r);
}

void abs_MPZ()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_abs(r, a);
  MPZ R = 12345;
  MPZ::abs(R);
  if (str(R) != str(r)) fail("abs(const MPZ &)");
  mpz_clear(a);
  mpz_clear(r);
}

void abs()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_abs(a, a);
  MPZ A = 12345;
  A.abs();
  if (str(A) != str(a)) fail("abs()");
  mpz_clear(a);
};

void tdiv_q_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_tdiv_q(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = MPZ::tdiv_q(A, B);
  if (str(R) != str(r)) fail("tdiv_q(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void operator_divide_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_tdiv_q(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = A / B;
  if (str(R) != str(r)) fail("operator /(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void tdiv_q_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_tdiv_q(a, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A.tdiv_q(B);
  if (str(A) != str(a)) fail("tdiv_q(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void operator_divideequals_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_tdiv_q(a, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A /= B;
  if (str(A) != str(a)) fail("operator /=(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void tdiv_r_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_tdiv_r(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = MPZ::tdiv_r(A, B);
  if (str(R) != str(r)) fail("tdiv_r(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void tdiv_r_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_tdiv_r(a, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A.tdiv_r(B);
  if (str(A) != str(a)) fail("tdiv_r(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void tdiv_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 98765);
  unsigned long int r = mpz_tdiv_ui(a, 12345);
  MPZ A = 98765;
  unsigned long int R = A.tdiv(12345);
  if (R != r) fail("tdiv(unsigned long int)");
  mpz_clear(a);
}

void fdiv_q_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_fdiv_q(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = MPZ::fdiv_q(A, B);
  if (str(R) != str(r)) fail("fdiv_q(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void fdiv_q_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_fdiv_q(a, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A.fdiv_q(B);
  if (str(A) != str(a)) fail("fdiv_q(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void fdiv_r_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_fdiv_r(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = MPZ::fdiv_r(A, B);
  if (str(R) != str(r)) fail("fdiv_r(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void fdiv_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 98765);
  unsigned long int r = mpz_fdiv_ui(a, 12345);
  MPZ A = 98765;
  unsigned long int R = A.fdiv(12345);
  if (R != r) fail("fdiv(unsigned long int)");
  mpz_clear(a);
}

void cdiv_q_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_cdiv_q(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = MPZ::cdiv_q(A, B);
  if (str(R) != str(r)) fail("cdiv_q(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void cdiv_q_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_cdiv_q(a, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A.cdiv_q(B);
  if (str(A) != str(a)) fail("cdiv_q(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void cdiv_r_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_cdiv_r(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = MPZ::cdiv_r(A, B);
  if (str(R) != str(r)) fail("cdiv_r(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void cdiv_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 98765);
  unsigned long int r = mpz_cdiv_ui(a, 12345);
  MPZ A = 98765;
  unsigned long int R = A.cdiv(12345);
  if (R != r) fail("cdiv(unsigned long int)");
  mpz_clear(a);
}

void mod_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_cdiv_q(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = MPZ::cdiv_q(A, B);
  if (str(R) != str(r)) fail("cdiv_q(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void operator_modulo_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_mod(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = A % B;
  if (str(R) != str(r)) fail("operator %%(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void mod_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_mod(a, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A.mod(B);
  if (str(A) != str(a)) fail("mod(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void operator_moduloequals_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_mod(a, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A %= B;
  if (str(A) != str(a)) fail("operator %%=(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void divexact_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 98765*12345);
  mpz_init(r);
  mpz_divexact(r, a, b);
  MPZ A = 98765;
  MPZ B = 98765*12345;
  MPZ R = MPZ::divexact(A, B);
  if (str(R) != str(r)) fail("divexact(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void divexact_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 98765*12345);
  mpz_divexact(a, a, b);
  MPZ A = 98765;
  MPZ B = 98765*12345;
  A.divexact(B);
  if (str(A) != str(a)) fail("divexact(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void tdiv_q_2exp_MPZ_unsigned_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 98765);
  mpz_init(r);
  mpz_tdiv_q_2exp(r, a, 5);
  MPZ A = 98765;
  MPZ R = MPZ::tdiv_q_2exp(A, 5);
  if (str(R) != str(r)) fail("tdiv_q_2exp(const MPZ &, unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void operator_shr_MPZ_unsigned_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 98765);
  mpz_init(r);
  mpz_tdiv_q_2exp(r, a, 5);
  MPZ A = 98765;
  MPZ R = A >> 5;
  if (str(R) != str(r)) fail("operator >>(unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void tdiv_q_2exp_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 98765);
  mpz_tdiv_q_2exp(a, a, 5);
  MPZ A = 98765;
  A.tdiv_q_2exp(5);
  if (str(A) != str(a)) fail("tdiv_q_2exp(unsigned long int)");
  mpz_clear(a);
}

void tdiv_operator_shrequals_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 98765);
  mpz_tdiv_q_2exp(a, a, 5);
  MPZ A = 98765;
  A >>= 5;
  if (str(A) != str(a)) fail("operator >>=(unsigned long int)");
  mpz_clear(a);
}

void fdiv_q_2exp_MPZ_unsigned_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 98765);
  mpz_init(r);
  mpz_fdiv_q_2exp(r, a, 5);
  MPZ A = 98765;
  MPZ R = MPZ::fdiv_q_2exp(A, 5);
  if (str(R) != str(r)) fail("fdiv_q_2exp(const MPZ &, unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void fdiv_q_2exp_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 98765);
  mpz_fdiv_q_2exp(a, a, 5);
  MPZ A = 98765;
  A.fdiv_q_2exp(5);
  if (str(A) != str(a)) fail("fdiv_q_2exp(unsigned long int)");
  mpz_clear(a);
}

void powm_MPZ_MPZ_MPZ()
{
  mpz_t a, b, c, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 5);
  mpz_init_set_ui(c, 12345);
  mpz_init(r);
  mpz_powm(r, a, b, c);
  MPZ A = 98765;
  MPZ B = 5;
  MPZ C = 12345;
  MPZ R = MPZ::powm(A, B, C);
  if (str(R) != str(r)) fail("powm(const MPZ &, const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(c);
  mpz_clear(r);
}

void powm_MPZ_unsigned_long_int_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_powm_ui(r, a, 5, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = MPZ::powm(A, 5, B);
  if (str(R) != str(r)) fail("powm(const MPZ &, unsigned long int, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void powm_MPZ_MPZ()
{
  mpz_t a, b, c;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 5);
  mpz_init_set_ui(c, 12345);
  mpz_powm(a, a, b, c);
  MPZ A = 98765;
  MPZ B = 5;
  MPZ C = 12345;
  A.powm(B, C);
  if (str(A) != str(a)) fail("powm(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(c);
}

void powm_unsigned_long_int_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_powm_ui(a, a, 5, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A.powm(5, B);
  if (str(A) != str(a)) fail("powm(unsigned long int, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void pow_MPZ_unsigned_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 98765);
  mpz_init(r);
  mpz_pow_ui(r, a, 5);
  MPZ A = 98765;
  MPZ R = MPZ::pow(A, 5);
  if (str(R) != str(r)) fail("pow(unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void pow_unsigned_long_int_unsigned_long_int()
{
  mpz_t r;
  mpz_init(r);
  mpz_ui_pow_ui(r, 98765, 5);
  MPZ R = MPZ::pow(98765, 5);
  if (str(R) != str(r)) fail("pow(unsigned long int, unsigned long int)");
  mpz_clear(r);
}

void pow_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 98765);
  mpz_pow_ui(a, a, 5);
  MPZ A = 98765;
  A.pow(5);
  if (str(A) != str(a)) fail("pow(unsigned long int)");
  mpz_clear(a);
}

void root_MPZ_unsigned_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 98765);
  mpz_init(r);
  mpz_root(r, a, 3);
  MPZ A = 98765;
  MPZ R = MPZ::root(A, 3);
  if (str(R) != str(r)) fail("root(const MPZ &, unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void root_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 98765);
  mpz_root(a, a, 3);
  MPZ A = 98765;
  A.root(3);
  if (str(A) != str(a)) fail("root(unsigned long int)");
  mpz_clear(a);
}

void sqrt_MPZ()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 98765);
  mpz_init(r);
  mpz_sqrt(r, a);
  MPZ A = 98765;
  MPZ R = MPZ::sqrt(A);
  if (str(R) != str(r)) fail("sqrt(const MPZ &)");
  mpz_clear(a);
  mpz_clear(r);
}

void sqrt()
{
  mpz_t a;
  mpz_init_set_ui(a, 98765);
  mpz_sqrt(a, a);
  MPZ A = 98765;
  A.sqrt();
  if (str(A) != str(a)) fail("sqrt()");
  mpz_clear(a);
}

void perfect_power_p()
{
  MPZ A = MPZ::pow(98765, 123);
  if (!A.perfect_power_p()) fail("perfect_power_p()");
  A += 5;
  if (A.perfect_power_p()) fail("perfect_power_p()");
}

void perfect_square_p()
{
  MPZ A = MPZ::pow(98765, 2);
  if (!A.perfect_square_p()) fail("perfect_square_p()");
  A += 5;
  if (A.perfect_square_p()) fail("perfect_square_p()");
}

void probab_prime_p()
{
  MPZ A = "99994742993377"; // largest Carmichael < 10^14
  if (A.probab_prime_p(5)) fail("probab_prime_p()");
  MPZ B = "99999999999973"; // largest prime < 10^14
  if (!B.probab_prime_p(5)) fail("probab_prime_p()");
}

void nextprime_MPZ()
{
  mpz_t a, r;
  mpz_init_set_str(a, "99999999999973", 10);
  mpz_init(r);
  mpz_nextprime(r, a);
  MPZ A = "99999999999973";
  MPZ R = MPZ::nextprime(A);
  if (str(R) != str(r)) fail("nextprime(const MPZ &)");
  mpz_clear(a);
  mpz_clear(r);
}

void nextprime()
{
  mpz_t a;
  mpz_init_set_str(a, "99999999999973", 10);
  mpz_nextprime(a, a);
  MPZ A = "99999999999973";
  A.nextprime();
  if (str(A) != str(a)) fail("nextprime(const MPZ &)");
  mpz_clear(a);
}

void gcd_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_gcd(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = MPZ::gcd(A, B);
  if (str(R) != str(r)) fail("gcd(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void gcd_MPZ_unsigned_long_int()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 98765);
  mpz_init(r);
  mpz_gcd_ui(r, a, 12345);
  MPZ A = 98765;
  MPZ R = MPZ::gcd(A, 12345);
  if (str(R) != str(r)) fail("gcd(const MPZ &, unsigned long int)");
  mpz_clear(a);
  mpz_clear(r);
}

void gcd_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_gcd(a, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A.gcd(B);
  if (str(A) != str(a)) fail("gcd(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void gcd_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 98765);
  unsigned long int r = mpz_gcd_ui(a, a, 12345);
  MPZ A = 98765;
  unsigned long int R = A.gcd(12345);
  if (R != r) fail("gcd(unsigned long int)");
  mpz_clear(a);
}

void lcm_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_lcm(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = MPZ::lcm(A, B);
  if (str(R) != str(r)) fail("lcm(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void lcm_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_lcm(a, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A.lcm(B);
  if (str(A) != str(a)) fail("lcm(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void invert_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 3235368842UL);
  mpz_init_set_ui(b, 3422090173UL);
  mpz_init(r);
  mpz_invert(r, a, b);
  MPZ A = 3235368842UL;
  MPZ B = 3422090173UL;
  MPZ R = MPZ::invert(A, B);
  if (str(R) != str(r)) fail("invert(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void invert_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 3235368842UL);
  mpz_init_set_ui(b, 3422090173UL);
  mpz_invert(a, a, b);
  MPZ A = 3235368842UL;
  MPZ B = 3422090173UL;
  A.invert(B);
  if (str(A) != str(a)) fail("invert(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void jacobi_MPZ_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  int r = mpz_jacobi(a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  int R = MPZ::jacobi(A, B);
  if (R != r) fail("jacobi(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void jacobi_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  int r = mpz_jacobi(a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  int R = A.jacobi(B);
  if (R != r) fail("jacobi(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void legendre_MPZ_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  int r = mpz_legendre(a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  int R = MPZ::legendre(A, B);
  if (R != r) fail("legendre(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void legendre_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  int r = mpz_legendre(a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  int R = A.legendre(B);
  if (R != r) fail("legendre(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void kronecker_signed_long_int_MPZ()
{
  mpz_t b;
  mpz_init_set_ui(b, 12345);
  int r = mpz_si_kronecker(LONG_MIN, b);
  MPZ B = 12345;
  int R = MPZ::kronecker(LONG_MIN, B);
  if (R != r) fail("kronecker(signed long int, const MPZ &)");
  mpz_clear(b);
}

void kronecker_signed_int_MPZ()
{
  mpz_t b;
  mpz_init_set_ui(b, 12345);
  int r = mpz_si_kronecker(INT_MIN, b);
  MPZ B = 12345;
  int R = MPZ::kronecker(INT_MIN, B);
  if (R != r) fail("kronecker(signed int, const MPZ &)");
  mpz_clear(b);
}

void kronecker_unsigned_long_int_MPZ()
{
  mpz_t b;
  mpz_init_set_ui(b, 12345);
  int r = mpz_ui_kronecker(ULONG_MAX, b);
  MPZ B = 12345;
  int R = MPZ::kronecker(ULONG_MAX, B);
  if (R != r) fail("kronecker(unsigned long int, const MPZ &)");
  mpz_clear(b);
}

void kronecker_unsigned_int_MPZ()
{
  mpz_t b;
  mpz_init_set_ui(b, 12345);
  int r = mpz_ui_kronecker(UINT_MAX, b);
  MPZ B = 12345;
  int R = MPZ::kronecker(UINT_MAX, B);
  if (R != r) fail("kronecker(unsigned int, const MPZ &)");
  mpz_clear(b);
}

void kronecker_MPZ_signed_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 98765);
  int r = mpz_kronecker_si(a, LONG_MIN);
  MPZ A = 98765;
  int R = MPZ::kronecker(A, LONG_MIN);
  if (R != r) fail("kronecker(const MPZ &, signed long int)");
  mpz_clear(a);
}

void kronecker_MPZ_signed_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 98765);
  int r = mpz_kronecker_si(a, INT_MIN);
  MPZ A = 98765;
  int R = MPZ::kronecker(A, INT_MIN);
  if (R != r) fail("kronecker(const MPZ &, signed int)");
  mpz_clear(a);
}

void kronecker_MPZ_unsigned_long_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 98765);
  int r = mpz_kronecker_ui(a, ULONG_MAX);
  MPZ A = 98765;
  int R = MPZ::kronecker(A, ULONG_MAX);
  if (R != r) fail("kronecker(const MPZ &, unsigned long int)");
  mpz_clear(a);
}

void kronecker_MPZ_unsigned_int()
{
  mpz_t a;
  mpz_init_set_ui(a, 98765);
  int r = mpz_kronecker_ui(a, UINT_MAX);
  MPZ A = 98765;
  int R = MPZ::kronecker(A, UINT_MAX);
  if (R != r) fail("kronecker(const MPZ &, unsigned int)");
  mpz_clear(a);
}

void remove_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 987654321);
  mpz_init_set_ui(b, 3);
  mpz_init(r);
  mpz_remove(r, a, b);
  MPZ A = 987654321;
  MPZ B = 3;
  MPZ R = MPZ::remove(A, B);
  if (str(R) != str(r)) fail("remove(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void remove_MPZ()
{
  mpz_t a, b;
  mpz_init_set_ui(a, 987654321);
  mpz_init_set_ui(b, 3);
  mpz_remove(a, a, b);
  MPZ A = 987654321;
  MPZ B = 3;
  A.remove(B);
  if (str(A) != str(a)) fail("remove(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void fac()
{
  mpz_t r;
  mpz_init(r);
  mpz_fac_ui(r, 12345);
  MPZ R = MPZ::fac(12345);
  if (str(R) != str(r)) fail("fac(unsigned long int)");
  mpz_clear(r);
}

void bin_MPZ_unsigned_long_int()
{
  mpz_t n, r;
  mpz_init_set_ui(n, 98765);
  mpz_init(r);
  mpz_bin_ui(r, n, 12345);
  MPZ N = 98765;
  MPZ R = MPZ::bin(N, 12345);
  if (str(R) != str(r)) fail("bin(const MPZ &, unsigned long int)");
  mpz_clear(n);
  mpz_clear(r);
}

void bin_unsigned_long_int_unsigned_long_int()
{
  mpz_t r;
  mpz_init(r);
  mpz_bin_uiui(r, 98765, 12345);
  MPZ R = MPZ::bin(98765, 12345);
  if (str(R) != str(r)) fail("bin(unsigned long int, unsigned long int)");
  mpz_clear(r);
}

void fib()
{
  mpz_t r;
  mpz_init(r);
  mpz_fib_ui(r, 12345);
  MPZ R = MPZ::fib(12345);
  if (str(R) != str(r)) fail("fib(unsigned long int)");
  mpz_clear(r);
}

void cmp_MPZ_MPZ()
{
  if (MPZ::cmp(MPZ(12345), MPZ(98765)) >= 0) fail("cmp(const MPZ &, const MPZ &)");
  if (MPZ::cmp(MPZ(12345), MPZ(12345)) != 0) fail("cmp(const MPZ &, const MPZ &)");
  if (MPZ::cmp(MPZ(98765), MPZ(12345)) <= 0) fail("cmp(const MPZ &, const MPZ &)");
}

void cmp_MPZ_unsigned_long_int()
{
  if (MPZ::cmp(MPZ(12345), 98765UL) >= 0) fail("cmp(const MPZ &, unsigned long int)");
  if (MPZ::cmp(MPZ(12345), 12345UL) != 0) fail("cmp(const MPZ &, unsigned long int)");
  if (MPZ::cmp(MPZ(98765), 12345UL) <= 0) fail("cmp(const MPZ &, unsigned long int)");
}

void cmp_MPZ_signed_long_int()
{
  if (MPZ::cmp(MPZ(12345), 98765L) >= 0) fail("cmp(const MPZ &, signed long int)");
  if (MPZ::cmp(MPZ(12345), 12345L) != 0) fail("cmp(const MPZ &, signed long int)");
  if (MPZ::cmp(MPZ(98765), 12345L) <= 0) fail("cmp(const MPZ &, signed long int)");
}

void and_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_and(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = MPZ::and(A, B);
  if (str(R) != str(r)) fail("and(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void operator_and_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_and(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = A & B;
  if (str(R) != str(r)) fail("operator &(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void and_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_and(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A.and(B);
  if (str(A) != str(r)) fail("and(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void operator_andequals_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_and(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A &= B;
  if (str(A) != str(r)) fail("operator &=(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void ior_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_ior(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = MPZ::ior(A, B);
  if (str(R) != str(r)) fail("ior(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void operator_or_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_ior(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = A | B;
  if (str(R) != str(r)) fail("operator |(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void ior_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_ior(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A.ior(B);
  if (str(A) != str(r)) fail("ior(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void operator_orequals_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_ior(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A |= B;
  if (str(A) != str(r)) fail("operator |=(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void xor_MPZ_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_xor(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = MPZ::xor(A, B);
  if (str(R) != str(r)) fail("xor(const MPZ &, const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void operator_xor_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_xor(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  MPZ R = A ^ B;
  if (str(R) != str(r)) fail("operator ^(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void xor_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_xor(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A.xor(B);
  if (str(A) != str(r)) fail("xor(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void operator_xorequals_MPZ()
{
  mpz_t a, b, r;
  mpz_init_set_ui(a, 98765);
  mpz_init_set_ui(b, 12345);
  mpz_init(r);
  mpz_xor(r, a, b);
  MPZ A = 98765;
  MPZ B = 12345;
  A ^= B;
  if (str(A) != str(r)) fail("operator ^=(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
  mpz_clear(r);
}

void com_MPZ()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_com(r, a);
  MPZ A = 12345;
  MPZ R = MPZ::com(A);
  if (str(R) != str(r)) fail("com(const MPZ &)");
  mpz_clear(a);
  mpz_clear(r);
}

void com()
{
  mpz_t a;
  mpz_init_set_ui(a, 12345);
  mpz_com(a, a);
  MPZ A = 12345;
  A.com();
  if (str(A) != str(a)) fail("com()");
  mpz_clear(a);
}

void operator_not()
{
  mpz_t a, r;
  mpz_init_set_ui(a, 12345);
  mpz_init(r);
  mpz_com(r, a);
  MPZ A = 12345;
  MPZ R = ~A;
  if (str(R) != str(r)) fail("operator ~()");
  mpz_clear(a);
  mpz_clear(r);
}

void popcount()
{
  mpz_t a;
  mpz_init_set_str(a, M727, 10);
  int r = mpz_popcount(a);
  MPZ A = M727;
  int R = A.popcount();
  if (R != r) fail("popcount()");
  mpz_clear(a);
}

void hamdist()
{
  mpz_t a, b;
  mpz_init_set_str(a, c0, 10);
  mpz_init_set_str(b, M727, 10);
  int r = mpz_hamdist(a, b);
  MPZ A = c0;
  MPZ B = M727;
  int R = A.hamdist(B);
  if (R != r) fail("hamdist(const MPZ &)");
  mpz_clear(a);
  mpz_clear(b);
}

void scan0()
{
  mpz_t a;
  mpz_init_set_str(a, c0, 10);
  MPZ A = c0;
  for (size_t i = 0; i < mpz_size(a); i++) {
    int r = mpz_scan0(a, i);
    int R = A.scan0(i);
    if (R != r) fail("scan0(unsigned long int)");
  }
  mpz_clear(a);
}

void scan1()
{
  mpz_t a;
  mpz_init_set_str(a, c0, 10);
  MPZ A = c0;
  for (size_t i = 0; i < mpz_size(a); i++) {
    int r = mpz_scan1(a, i);
    int R = A.scan1(i);
    if (R != r) fail("scan1(unsigned long int)");
  }
  mpz_clear(a);
}

void setbit()
{
  mpz_t a;
  mpz_init(a);
  mpz_setbit(a, 76);
  MPZ A;
  A.setbit(76);
  if (str(A) != str(a)) fail("setbit(unisnged long int)");
  mpz_clear(a);
}

void clrbit()
{
  mpz_t a;
  mpz_init_set_si(a, -1);
  mpz_clrbit(a, 76);
  MPZ A = -1;
  A.clrbit(76);
  if (str(A) != str(a)) fail("clrbit(unisnged long int)");
  mpz_clear(a);
}

void tstbit()
{
  mpz_t a;
  mpz_init_set_str(a, c0, 10);
  MPZ A = c0;
  for (size_t i = 0; i < mpz_size(a); i++) {
    int r = mpz_tstbit(a, i);
    int R = A.tstbit(i);
    if (R != r) fail("tstbit(unsigned long int)");
  }
  mpz_clear(a);
}

void out_str()
{
  FILE *f = fopen("gmpcpp.test$", "w+");
  if (f == NULL) {
    fail("test_out_str()");
    return;
  }
  MPZ A = M727;
  size_t r = A.out_str(f);
  if (r != strlen(M727)) fail("test_out_str()");
  fseek(f, 0, SEEK_SET);
  MPZ B = MPZ::inp_str(f);
  if (A != B) fail("test_inp_str()");
  fclose(f);
  unlink("gmpcpp.test$");
}

void inp_str()
{
  out_str();
}

void out_raw()
{
  FILE *f = fopen("gmpcpp.test$", "w+");
  if (f == NULL) {
    fail("test_out_raw()");
    return;
  }
  MPZ A = M727;
  A.out_raw(f);
  fseek(f, 0, SEEK_SET);
  MPZ B = MPZ::inp_raw(f);
  if (A != B) fail("test_inp_raw()");
  fclose(f);
  unlink("gmpcpp.test$");
}

void inp_raw()
{
  out_raw();
}

void fits_ulong_p()
{
  MPZ A = ULONG_MAX;
  if (!A.fits_ulong_p()) fail("fits_ulong_p()");
  A++;
  if (A.fits_ulong_p()) fail("fits_ulong_p()");
}

void fits_slong_p()
{
  MPZ A = LONG_MIN;
  if (!A.fits_slong_p()) fail("fits_slong_p()");
  A--;
  if (A.fits_slong_p()) fail("fits_slong_p()");
}

void fits_uint_p()
{
  MPZ A = UINT_MAX;
  if (!A.fits_uint_p()) fail("fits_uint_p()");
  A++;
  if (A.fits_uint_p()) fail("fits_uint_p()");
}

void fits_sint_p()
{
  MPZ A = INT_MIN;
  if (!A.fits_sint_p()) fail("fits_sint_p()");
  A--;
  if (A.fits_sint_p()) fail("fits_sint_p()");
}

void fits_ushort_p()
{
  MPZ A = USHRT_MAX;
  if (!A.fits_ushort_p()) fail("fits_ushort_p()");
  A++;
  if (A.fits_ushort_p()) fail("fits_ushort_p()");
}

void fits_sshort_p()
{
  MPZ A = SHRT_MIN;
  if (!A.fits_sshort_p()) fail("fits_sshort_p()");
  A--;
  if (A.fits_sshort_p()) fail("fits_sshort_p()");
}

void odd()
{
  MPZ A = c0;
  if (A.odd()) fail("odd()");
  A++;
  if (!A.odd()) fail("odd()");
}

void even()
{
  MPZ A = c0;
  if (!A.even()) fail("even()");
  A++;
  if (A.even()) fail("even()");
}

void size()
{
  mpz_t a;
  mpz_init_set_str(a, c0, 10);
  int r = mpz_size(a);
  MPZ A = c0;
  int R = A.size();
  if (R != r) fail("size()");
}

void sizeinbase()
{
  mpz_t a;
  mpz_init_set_str(a, M727, 10);
  int r = mpz_sizeinbase(a, 36);
  MPZ A = M727;
  int R = A.sizeinbase(36);
  if (R != r) fail("sizeinbase()");
}

void operator_put()
{
  std::fstream f("gmpcpp.test$", std::ios::in|std::ios::out);
  if (!f) {
    fail("operator_put()");
    return;
  }
  MPZ A = M727;
  f << A;
  f.seekp(0);
  MPZ B;
  f >> B;
  if (A != B) fail("operator_put()");
  f.close();
  unlink("gmpcpp.test$");
}

void operator_get()
{
  operator_put();
}

} // namespace test_MPZ

namespace test_MPQ {

} // namespace test_MPQ

namespace test_MPF {

} // namespace test_MPF

int main(int, char *[])
{
  MPZ A = 5;
  MPZ B = 7 - A;
  RANDSTATE r;
  MPZ C = r.urandomb(64);
  cout << C << endl;

  // Assignment Functions
  test_MPZ::set_MPZ();
  test_MPZ::set_unsigned_long_int();
  test_MPZ::set_unsigned_int();
  test_MPZ::set_signed_long_int();
  test_MPZ::set_signed_int();
  test_MPZ::set_double();
  //test_MPZ::set_MPQ();
  //test_MPZ::set_MPF();
  test_MPZ::set_char();
  test_MPZ::operator_eq_MPZ();
  test_MPZ::operator_eq_unsigned_long_int();
  test_MPZ::operator_eq_unsigned_int();
  test_MPZ::operator_eq_signed_long_int();
  test_MPZ::operator_eq_signed_int();
  test_MPZ::operator_eq_double();
  //test_MPZ::operator_eq_MPQ();
  //test_MPZ::operator_eq_MPF();
  test_MPZ::operator_eq_char();
  test_MPZ::swap();

  // Combined Initialization and Assignment Functions
  test_MPZ::constructor_MPZ();
  test_MPZ::constructor_unsigned_long_int();
  test_MPZ::constructor_unsigned_int();
  test_MPZ::constructor_signed_long_int();
  test_MPZ::constructor_signed_int();
  test_MPZ::constructor_double();
  test_MPZ::constructor_char();

  // Conversion Functions
  test_MPZ::getlimbn();
  test_MPZ::get_ui();
  test_MPZ::get_si();
  test_MPZ::get_d();
  test_MPZ::get_str();

  // Arithmetic Functions
  test_MPZ::add_MPZ_MPZ();
  test_MPZ::add_MPZ_unsigned_long_int();
  test_MPZ::operator_plus_MPZ();
  test_MPZ::operator_plus_unsigned_long_int();
  test_MPZ::add_MPZ();
  test_MPZ::add_unsigned_long_int();
  test_MPZ::operator_plusequals_MPZ();
  test_MPZ::operator_plusequals_unsigned_long_int();
  test_MPZ::operator_increment_prefix();
  test_MPZ::operator_increment_postfix();
  test_MPZ::sub_MPZ_MPZ();
  test_MPZ::sub_MPZ_unsigned_long_int();
  test_MPZ::operator_minus_MPZ();
  test_MPZ::operator_minus_unsigned_long_int();
  test_MPZ::sub_MPZ();
  test_MPZ::sub_unsigned_long_int();
  test_MPZ::operator_minusequals_MPZ();
  test_MPZ::operator_minusequals_unsigned_long_int();
  test_MPZ::operator_decrement_prefix();
  test_MPZ::operator_decrement_postfix();
  test_MPZ::mul_MPZ_MPZ();
  test_MPZ::mul_MPZ_unsigned_long_int();
  test_MPZ::mul_MPZ_unsigned_int();
  test_MPZ::mul_MPZ_signed_long_int();
  test_MPZ::mul_MPZ_signed_int();
  test_MPZ::operator_times_MPZ();
  test_MPZ::operator_times_unsigned_long_int();
  test_MPZ::operator_times_unsigned_int();
  test_MPZ::operator_times_signed_long_int();
  test_MPZ::operator_times_signed_int();
  test_MPZ::mul_MPZ();
  test_MPZ::mul_unsigned_long_int();
  test_MPZ::mul_unsigned_int();
  test_MPZ::mul_signed_long_int();
  test_MPZ::mul_signed_int();
  test_MPZ::operator_timesequals_MPZ();
  test_MPZ::operator_timesequals_unsigned_long_int();
  test_MPZ::operator_timesequals_unsigned_int();
  test_MPZ::operator_timesequals_signed_long_int();
  test_MPZ::operator_timesequals_signed_int();
  test_MPZ::addmul();
  test_MPZ::mul_2exp_MPZ_unsigned_long_int();
  test_MPZ::operator_shl_unsigned_long_int();
  test_MPZ::mul_2exp_unsigned_long_int();
  test_MPZ::operator_shlequals_unsigned_long_int();
  test_MPZ::neg_MPZ();
  test_MPZ::neg();
  test_MPZ::operator_minus();
  test_MPZ::abs_MPZ();
  test_MPZ::abs();

  // Division Functions
  test_MPZ::tdiv_q_MPZ_MPZ();
  test_MPZ::operator_divide_MPZ();
  test_MPZ::tdiv_q_MPZ();
  test_MPZ::operator_divideequals_MPZ();
  test_MPZ::tdiv_r_MPZ_MPZ();
  test_MPZ::tdiv_r_MPZ();
  test_MPZ::tdiv_unsigned_long_int();
  test_MPZ::fdiv_q_MPZ_MPZ();
  test_MPZ::fdiv_q_MPZ();
  test_MPZ::fdiv_r_MPZ_MPZ();
  test_MPZ::fdiv_unsigned_long_int();
  test_MPZ::cdiv_q_MPZ_MPZ();
  test_MPZ::cdiv_q_MPZ();
  test_MPZ::cdiv_r_MPZ_MPZ();
  test_MPZ::cdiv_unsigned_long_int();
  test_MPZ::mod_MPZ_MPZ();
  test_MPZ::operator_modulo_MPZ();
  test_MPZ::mod_MPZ();
  test_MPZ::operator_moduloequals_MPZ();
  test_MPZ::divexact_MPZ_MPZ();
  test_MPZ::divexact_MPZ();
  test_MPZ::tdiv_q_2exp_MPZ_unsigned_long_int();
  test_MPZ::tdiv_q_2exp_unsigned_long_int();
  test_MPZ::fdiv_q_2exp_MPZ_unsigned_long_int();
  test_MPZ::fdiv_q_2exp_unsigned_long_int();

  // Exponentiation Functions
  test_MPZ::powm_MPZ_MPZ_MPZ();
  test_MPZ::powm_MPZ_unsigned_long_int_MPZ();
  test_MPZ::powm_MPZ_MPZ();
  test_MPZ::powm_unsigned_long_int_MPZ();
  test_MPZ::pow_MPZ_unsigned_long_int();
  test_MPZ::pow_unsigned_long_int_unsigned_long_int();
  test_MPZ::pow_unsigned_long_int();

  // Root Extraction Functions
  test_MPZ::root_MPZ_unsigned_long_int();
  test_MPZ::root_unsigned_long_int();
  test_MPZ::sqrt_MPZ();
  test_MPZ::sqrt();
  test_MPZ::perfect_power_p();
  test_MPZ::perfect_square_p();

  // Number Theoretic Functions
  test_MPZ::probab_prime_p();
  test_MPZ::nextprime_MPZ();
  test_MPZ::nextprime();
  test_MPZ::gcd_MPZ_MPZ();
  test_MPZ::gcd_MPZ_unsigned_long_int();
  test_MPZ::gcd_MPZ();
  test_MPZ::gcd_unsigned_long_int();
  test_MPZ::lcm_MPZ_MPZ();
  test_MPZ::lcm_MPZ();
  test_MPZ::invert_MPZ_MPZ();
  test_MPZ::invert_MPZ();
  test_MPZ::jacobi_MPZ_MPZ();
  test_MPZ::jacobi_MPZ();
  test_MPZ::legendre_MPZ_MPZ();
  test_MPZ::legendre_MPZ();
  test_MPZ::kronecker_signed_long_int_MPZ();
  test_MPZ::kronecker_signed_int_MPZ();
  test_MPZ::kronecker_unsigned_long_int_MPZ();
  test_MPZ::kronecker_unsigned_int_MPZ();
  test_MPZ::kronecker_MPZ_signed_long_int();
  test_MPZ::kronecker_MPZ_signed_int();
  test_MPZ::kronecker_MPZ_unsigned_long_int();
  test_MPZ::kronecker_MPZ_unsigned_int();
  test_MPZ::remove_MPZ_MPZ();
  test_MPZ::remove_MPZ();
  test_MPZ::fac();
  test_MPZ::bin_MPZ_unsigned_long_int();
  test_MPZ::bin_unsigned_long_int_unsigned_long_int();
  test_MPZ::fib();

  // Comparison Functions
  test_MPZ::cmp_MPZ_MPZ();
  test_MPZ::cmp_MPZ_unsigned_long_int();
  test_MPZ::cmp_MPZ_signed_long_int();
  //test_MPZ::cmp_MPZ();
  //test_MPZ::cmp_unsigned_long_int();
  //test_MPZ::cmp_signed_long_int();
  //test_MPZ::operator_eq_MPZ();
  //test_MPZ::operator_eq_unsigned_long_int();
  //test_MPZ::operator_eq_signed_long_int();
  //test_MPZ::operator_ne_MPZ();
  //test_MPZ::operator_ne_unsigned_long_int();
  //test_MPZ::operator_ne_signed_long_int();
  //test_MPZ::operator_gt_MPZ();
  //test_MPZ::operator_gt_unsigned_long_int();
  //test_MPZ::operator_gt_signed_long_int();
  //test_MPZ::operator_ge_MPZ();
  //test_MPZ::operator_ge_unsigned_long_int();
  //test_MPZ::operator_ge_signed_long_int();
  //test_MPZ::operator_le_MPZ();
  //test_MPZ::operator_le_unsigned_long_int();
  //test_MPZ::operator_le_signed_long_int();
  //test_MPZ::operator_lt_MPZ();
  //test_MPZ::operator_lt_unsigned_long_int();
  //test_MPZ::operator_lt_signed_long_int();
  //test_MPZ::cmpabs_MPZ_MPZ();
  //test_MPZ::cmpabs_MPZ_unsigned_long_int();
  //test_MPZ::cmpabs_MPZ();
  //test_MPZ::cmpabs_unsigned_long_int();
  //test_MPZ::sgn();

  // Logical and Bit Manipulation Functions
  test_MPZ::and_MPZ_MPZ();
  test_MPZ::operator_and_MPZ();
  test_MPZ::and_MPZ();
  test_MPZ::operator_andequals_MPZ();
  test_MPZ::ior_MPZ_MPZ();
  test_MPZ::operator_or_MPZ();
  test_MPZ::ior_MPZ();
  test_MPZ::operator_orequals_MPZ();
  test_MPZ::xor_MPZ_MPZ();
  test_MPZ::operator_xor_MPZ();
  test_MPZ::xor_MPZ();
  test_MPZ::operator_xorequals_MPZ();
  test_MPZ::com_MPZ();
  test_MPZ::com();
  test_MPZ::operator_not();
  test_MPZ::popcount();
  test_MPZ::hamdist();
  test_MPZ::scan0();
  test_MPZ::scan1();
  test_MPZ::setbit();
  test_MPZ::clrbit();
  test_MPZ::tstbit();

  // Input and Output Functions
  test_MPZ::out_str();
  test_MPZ::inp_str();
  test_MPZ::out_raw();
  test_MPZ::inp_raw();

  // Miscellaneous Functions
  test_MPZ::fits_ulong_p();
  test_MPZ::fits_slong_p();
  test_MPZ::fits_uint_p();
  test_MPZ::fits_sint_p();
  test_MPZ::fits_ushort_p();
  test_MPZ::fits_sshort_p();
  test_MPZ::odd();
  test_MPZ::even();
  test_MPZ::size();
  test_MPZ::sizeinbase();

  test_MPZ::operator_put();
  test_MPZ::operator_get();

  if (failures > 0) {
    printf("%d tests failed\n", failures);
    return 1;
  }
  printf("all tests passed\n");
  return 0;
}
